@@ -25,7 +25,7 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import units
 
-from hyperv.i18n import _, _LW, _LE
+from hyperv.i18n import _, _LW
 from hyperv.nova import block_device_manager
 from hyperv.nova import constants
 from hyperv.nova import imagecache
@@ -46,59 +46,21 @@ class MigrationOps(object):
         self._imagecache = imagecache.ImageCache()
         self._block_dev_man = block_device_manager.BlockDeviceInfoManager()
 
-    def _migrate_disk_files(self, instance_name, disk_files, dest):
-        # TODO(mikal): it would be nice if this method took a full instance,
-        # because it could then be passed to the log messages below.
-        instance_path = self._pathutils.get_instance_dir(instance_name)
-        dest_path = self._pathutils.get_instance_dir(instance_name, dest)
+    def _move_vm_files(self, instance):
+        instance_path = self._pathutils.get_instance_dir(instance.name)
         revert_path = self._pathutils.get_instance_migr_revert_dir(
-            instance_name, remove_dir=True, create_dir=True)
+            instance.name, remove_dir=True, create_dir=True)
 
-        shared_storage = (self._pathutils.exists(dest_path) and
-                          self._pathutils.check_dirs_shared_storage(
-                              instance_path, dest_path))
-        try:
-            if shared_storage:
-                # Since source and target are the same, we copy the files to
-                # a temporary location before moving them into place.
-                # This applies when the migration target is the source host or
-                # when shared storage is used for the instance files.
-                dest_path = '%s_tmp' % instance_path
+        # copy the given instance's files to a _revert folder, as backup.
+        LOG.debug("Moving instance files to a revert path: %s",
+                  revert_path, instance=instance)
+        self._pathutils.move_folder_files(instance_path, revert_path)
 
-            self._pathutils.check_remove_dir(dest_path)
-            self._pathutils.makedirs(dest_path)
-
-            for disk_file in disk_files:
-                LOG.debug('Copying disk "%(disk_file)s" to '
-                          '"%(dest_path)s"',
-                          {'disk_file': disk_file, 'dest_path': dest_path})
-                self._pathutils.copy(disk_file, dest_path)
-
-            self._pathutils.move_folder_files(instance_path, revert_path)
-
-            if shared_storage:
-                self._pathutils.move_folder_files(dest_path, instance_path)
-        except Exception:
-            with excutils.save_and_reraise_exception():
-                self._cleanup_failed_disk_migration(instance_path, revert_path,
-                                                    dest_path)
-
-    def _cleanup_failed_disk_migration(self, instance_path,
-                                       revert_path, dest_path):
-        try:
-            if dest_path and self._pathutils.exists(dest_path):
-                self._pathutils.rmtree(dest_path)
-            if self._pathutils.exists(revert_path):
-                self._pathutils.move_folder_files(revert_path, instance_path)
-                self._pathutils.rmtree(revert_path)
-        except Exception as ex:
-            # Log and ignore this exception
-            LOG.exception(ex)
-            LOG.error(_LE("Cannot cleanup migration files"))
+        return revert_path
 
     def _check_target_flavor(self, instance, flavor):
         new_root_gb = flavor.root_gb
-        curr_root_gb = instance.root_gb
+        curr_root_gb = instance.flavor.root_gb
 
         if new_root_gb < curr_root_gb:
             raise exception.InstanceFaultRollback(
@@ -118,19 +80,14 @@ class MigrationOps(object):
         self._check_target_flavor(instance, flavor)
 
         self._vmops.power_off(instance, timeout, retry_interval)
+        instance_path = self._move_vm_files(instance)
 
-        (disk_files,
-         volume_drives) = self._vmutils.get_vm_storage_paths(instance.name)
+        self._vmops.destroy(instance, destroy_disks=True)
 
-        if disk_files:
-            self._migrate_disk_files(instance.name, disk_files, dest)
+        # return the instance's path location.
+        return instance_path
 
-        self._vmops.destroy(instance, destroy_disks=False)
-
-        # disk_info is not used
-        return ""
-
-    def confirm_migration(self, migration, instance, network_info):
+    def confirm_migration(self, context, migration, instance, network_info):
         LOG.debug("confirm_migration called", instance=instance)
 
         self._pathutils.get_instance_migr_revert_dir(instance.name,
@@ -158,33 +115,18 @@ class MigrationOps(object):
     def finish_revert_migration(self, context, instance, network_info,
                                 block_device_info=None, power_on=True):
         LOG.debug("finish_revert_migration called", instance=instance)
-
-        instance_name = instance.name
-        self._revert_migration_files(instance_name)
+        self._revert_migration_files(instance.name)
 
         image_meta = self._imagecache.get_image_details(context, instance)
         vm_gen = self._vmops.get_image_vm_generation(instance.uuid, image_meta)
-
-        self._block_dev_man.validate_and_update_bdi(instance, image_meta,
-                                                    vm_gen, block_device_info)
-        root_device = block_device_info['root_disk']
-
-        if root_device['type'] == constants.DISK:
-            root_vhd_path = self._pathutils.lookup_root_vhd_path(instance_name)
-            root_device['path'] = root_vhd_path
-            if not root_vhd_path:
-                base_vhd_path = self._pathutils.get_instance_dir(instance_name)
-                raise exception.DiskNotFound(location=base_vhd_path)
-
-        ephemerals = block_device_info['ephemerals']
-        self._check_ephemeral_disks(instance, ephemerals)
+        self._check_and_update_disks(context, instance, vm_gen, image_meta,
+                                     block_device_info)
 
         self._vmops.create_instance(context, instance, network_info,
-                                    root_device, block_device_info, vm_gen,
-                                    image_meta)
+                                    block_device_info, vm_gen, image_meta)
 
         self._check_and_attach_config_drive(instance, vm_gen)
-        self._vmops.set_boot_order(vm_gen, block_device_info, instance_name)
+        self._vmops.set_boot_order(instance.name, vm_gen, block_device_info)
         if power_on:
             self._vmops.power_on(instance, network_info=network_info)
 
@@ -257,23 +199,45 @@ class MigrationOps(object):
             self._vhdutils.reconnect_parent_vhd(diff_vhd_path,
                                                 base_vhd_path)
 
+    def _migrate_disks_from_source(self, migration, instance,
+                                   source_inst_dir):
+        source_inst_dir = self._pathutils.get_remote_path(
+            migration.source_compute, source_inst_dir)
+        inst_dir = self._pathutils.get_instance_dir(
+            instance.name, create_dir=True, remove_dir=True)
+
+        self._pathutils.copy_folder_files(source_inst_dir, inst_dir)
+
     def finish_migration(self, context, migration, instance, disk_info,
                          network_info, image_meta, resize_instance=False,
                          block_device_info=None, power_on=True):
         LOG.debug("finish_migration called", instance=instance)
+        self._migrate_disks_from_source(migration, instance, disk_info)
 
-        instance_name = instance.name
         vm_gen = self._vmops.get_image_vm_generation(instance.uuid, image_meta)
+        self._check_and_update_disks(context, instance, vm_gen, image_meta,
+                                     block_device_info,
+                                     resize_instance=resize_instance)
 
+        self._vmops.create_instance(context, instance, network_info,
+                                    block_device_info, vm_gen, image_meta)
+
+        self._check_and_attach_config_drive(instance, vm_gen)
+        self._vmops.set_boot_order(instance.name, vm_gen, block_device_info)
+        if power_on:
+            self._vmops.power_on(instance, network_info=network_info)
+
+    def _check_and_update_disks(self, context, instance, vm_gen, image_meta,
+                                block_device_info, resize_instance=False):
         self._block_dev_man.validate_and_update_bdi(instance, image_meta,
                                                     vm_gen, block_device_info)
         root_device = block_device_info['root_disk']
 
         if root_device['type'] == constants.DISK:
-            root_vhd_path = self._pathutils.lookup_root_vhd_path(instance_name)
+            root_vhd_path = self._pathutils.lookup_root_vhd_path(instance.name)
             root_device['path'] = root_vhd_path
             if not root_vhd_path:
-                base_vhd_path = self._pathutils.get_instance_dir(instance_name)
+                base_vhd_path = self._pathutils.get_instance_dir(instance.name)
                 raise exception.DiskNotFound(location=base_vhd_path)
 
             root_vhd_info = self._vhdutils.get_vhd_info(root_vhd_path)
@@ -283,21 +247,12 @@ class MigrationOps(object):
                                       src_base_disk_path)
 
             if resize_instance:
-                new_size = instance.root_gb * units.Gi
+                new_size = instance.flavor.root_gb * units.Gi
                 self._check_resize_vhd(root_vhd_path, root_vhd_info, new_size)
 
         ephemerals = block_device_info['ephemerals']
         self._check_ephemeral_disks(instance, ephemerals, resize_instance)
 
-        self._vmops.create_instance(context, instance, network_info,
-                                    root_device, block_device_info,
-                                    vm_gen, image_meta)
-
-        self._check_and_attach_config_drive(instance, vm_gen)
-        self._vmops.set_boot_order(vm_gen, block_device_info, instance_name)
-        if power_on:
-            self._vmops.power_on(instance, network_info=network_info)
-
     def _check_ephemeral_disks(self, instance, ephemerals,
                                resize_instance=False):
         instance_name = instance.name
@@ -312,8 +267,8 @@ class MigrationOps(object):
         elif sum(eph['size'] for eph in ephemerals) != new_eph_gb:
             # New ephemeral size is different from the original ephemeral size
             # and there are multiple ephemerals.
-            LOG.warn(_LW("Cannot resize multiple ephemeral disks for "
-                         "instance."), instance=instance)
+            LOG.warning(_LW("Cannot resize multiple ephemeral disks for "
+                            "instance."), instance=instance)
 
         for index, eph in enumerate(ephemerals):
             eph_name = "eph%s" % index
