@@ -183,13 +183,23 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
             mock.sentinel.instance, mock.sentinel.network_info,
             mock.sentinel.block_device_info, mock.sentinel.destroy_disks)
 
+    def test_cleanup(self):
+        self.driver.cleanup(
+            mock.sentinel.context, mock.sentinel.instance,
+            mock.sentinel.network_info, mock.sentinel.block_device_info,
+            mock.sentinel.destroy_disks, mock.sentinel.migrate_data,
+            mock.sentinel.destroy_vifs)
+
+        self.driver._vmops.unplug_vifs.assert_called_once_with(
+            mock.sentinel.instance, mock.sentinel.network_info)
+
     def test_get_info(self):
         self.driver.get_info(mock.sentinel.instance)
         self.driver._vmops.get_info.assert_called_once_with(
             mock.sentinel.instance)
 
     def test_attach_volume(self):
-        mock_instance = mock.MagicMock()
+        mock_instance = fake_instance.fake_instance_obj(self.context)
         self.driver.attach_volume(
             mock.sentinel.context, mock.sentinel.connection_info,
             mock_instance, mock.sentinel.mountpoint, mock.sentinel.disk_bus,
@@ -200,7 +210,7 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
             mock_instance.name)
 
     def test_detach_volume(self):
-        mock_instance = mock.MagicMock()
+        mock_instance = fake_instance.fake_instance_obj(self.context)
         self.driver.detach_volume(
             mock.sentinel.connection_info, mock_instance,
             mock.sentinel.mountpoint, mock.sentinel.encryption)
@@ -384,12 +394,18 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
             mock.sentinel.dest_check_data)
 
     def test_plug_vifs(self):
-        self.assertRaises(NotImplementedError, self.driver.plug_vifs,
-                          mock.sentinel.instance, mock.sentinel.network_info)
+        self.driver.plug_vifs(
+            mock.sentinel.instance, mock.sentinel.network_info)
+
+        self.driver._vmops.plug_vifs.assert_called_once_with(
+            mock.sentinel.instance, mock.sentinel.network_info)
 
     def test_unplug_vifs(self):
-        self.assertRaises(NotImplementedError, self.driver.unplug_vifs,
-                          mock.sentinel.instance, mock.sentinel.network_info)
+        self.driver.unplug_vifs(
+            mock.sentinel.instance, mock.sentinel.network_info)
+
+        self.driver._vmops.unplug_vifs.assert_called_once_with(
+            mock.sentinel.instance, mock.sentinel.network_info)
 
     def test_refresh_instance_security_rules(self):
         self.assertRaises(NotImplementedError,
@@ -412,10 +428,12 @@ class HyperVDriverTestCase(test_base.HyperVBaseTestCase):
 
     def test_confirm_migration(self):
         self.driver.confirm_migration(
+            mock.sentinel.context,
             mock.sentinel.migration, mock.sentinel.instance,
             mock.sentinel.network_info)
 
         self.driver._migrationops.confirm_migration.assert_called_once_with(
+            mock.sentinel.context,
             mock.sentinel.migration, mock.sentinel.instance,
             mock.sentinel.network_info)
 
